package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.bean.dto.ExamTaskDetailDto;
import com.qmth.distributed.print.business.bean.dto.ExamTaskDto;
import com.qmth.distributed.print.business.entity.ExamTask;
import com.qmth.distributed.print.business.entity.ExamTaskDetail;
import com.qmth.distributed.print.business.enums.ExamStatusEnum;
import com.qmth.distributed.print.business.mapper.ExamTaskDetailMapper;
import com.qmth.distributed.print.business.service.ExamTaskDetailService;
import com.qmth.distributed.print.business.service.ExamTaskService;
import com.qmth.distributed.print.common.enums.ExceptionResultEnum;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * <p>
 * 命题任务明细表（入库申请） 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class ExamTaskDetailServiceImpl extends ServiceImpl<ExamTaskDetailMapper, ExamTaskDetail> implements ExamTaskDetailService {

    @Autowired
    private ExamTaskService examTaskService;

    @Override
    public boolean saveExamTaskDetail(ExamTaskDetail examTaskDetail) {
        if (examTaskDetail.getExamTaskId() == null) {
            throw ExceptionResultEnum.ERROR.exception("命题任务ID不能为空");
        }
        if (StringUtils.isBlank(examTaskDetail.getOperateType())) {
            throw ExceptionResultEnum.ERROR.exception("保存类型不能为空");
        }
        // 提交时，校验
        if (ExamStatusEnum.SUBMIT.name().equals(examTaskDetail.getOperateType())) {
            if (StringUtils.isBlank(examTaskDetail.getPaperType())) {
                throw ExceptionResultEnum.ERROR.exception("试卷类型不能为空");
            }
            if (StringUtils.isBlank(examTaskDetail.getPaperAttachmentIds())) {
                throw ExceptionResultEnum.ERROR.exception("试卷ID不能为空");
            }
            if (examTaskDetail.getCardId() == null) {
                throw ExceptionResultEnum.ERROR.exception("题卡ID不能为空");
            }
            if (StringUtils.isBlank(examTaskDetail.getPaperConfirmAttachmentIds())) {
                throw ExceptionResultEnum.ERROR.exception("入库审核ID不能为空");
            }
            QueryWrapper<ExamTaskDetail> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(ExamTaskDetail::getExamTaskId, examTaskDetail.getExamTaskId());
            ExamTaskDetail detail = this.getOne(queryWrapper);
            if (detail != null) {
                examTaskDetail.setId(detail.getId());
            }
        }

        return this.saveOrUpdate(examTaskDetail);
    }

    @Override
    public boolean status(ExamTask examTask) {
        UpdateWrapper<ExamTask> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamTask::getStatus, examTask.getStatus()).eq(ExamTask::getId, examTask.getId());
        return this.examTaskService.update(updateWrapper);
    }

    @Override
    public boolean enable(ExamTaskDetail examTaskDetail) {
        UpdateWrapper<ExamTaskDetail> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamTaskDetail::getEnable, examTaskDetail.getEnable()).eq(ExamTaskDetail::getId, examTaskDetail.getId());
        return this.update(updateWrapper);
    }

}
