package com.qmth.distributed.print.api;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.distributed.print.business.bean.ApiUser;
import com.qmth.distributed.print.business.entity.SysRole;
import com.qmth.distributed.print.business.service.SysRoleService;
import com.qmth.distributed.print.business.util.RequestUtils;
import com.qmth.distributed.print.common.util.Result;
import com.qmth.distributed.print.common.util.ResultUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * <p>
 * 角色表 前端控制器
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Api(tags = "角色Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/${prefix.url.sys}/role")
@Aac(auth = BOOL.FALSE)
public class SysRoleController {

    @Autowired
    private SysRoleService sysRoleService;

    /**
     * 查询
     *
     * @param request
     * @param name
     * @param enable
     * @param pageNumber
     * @param pageSize
     * @return
     */
    @ApiOperation(value = "查询")
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    public Result list(HttpServletRequest request,
                       @RequestParam(value = "name", required = false) String name,
                       @RequestParam(value = "enable", required = false) Boolean enable,
                       @RequestParam(value = "pageNumber", required = true) Integer pageNumber,
                       @RequestParam(value = "pageSize", required = true) Integer pageSize) {
        ApiUser apiUser = RequestUtils.getApiUser(request);
        IPage<SysRole> userDtos = sysRoleService.list(apiUser.getSchoolId(), name, enable, pageNumber, pageSize);
        return ResultUtil.ok(userDtos);
    }

    /**
     * 新增/修改
     *
     * @param request
     * @param role
     * @return
     */
    @ApiOperation(value = "新增/修改")
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    public Result save(HttpServletRequest request, @RequestBody SysRole role) {
        ApiUser apiUser = RequestUtils.getApiUser(request);
        role.setSchoolId(apiUser.getSchoolId());
        role.setCreateId(role.getId() == null ? apiUser.getUserData().getId() : null);
        role.setUpdateId(role.getId() != null ? apiUser.getUserData().getId() : null);
        boolean isSuccess = sysRoleService.saveRole(role);
        return ResultUtil.ok(isSuccess);
    }

    /**
     * 启用/禁用
     *
     * @param role
     * @return
     */
    @ApiOperation(value = "启用/禁用")
    @RequestMapping(value = "/enable", method = RequestMethod.POST)
    public Result enable(@RequestBody SysRole role) {
        boolean isSuccess = sysRoleService.enable(role);
        return ResultUtil.ok(isSuccess);
    }

    /**
     * 删除
     *
     * @param request
     * @param id
     * @return
     */
    @ApiOperation(value = "删除")
    @RequestMapping(value = "/remove", method = RequestMethod.POST)
    public Result remove(HttpServletRequest request, Long id) {
        boolean isSuccess = sysRoleService.remove(id);
        return ResultUtil.ok(isSuccess);
    }

    /**
     * 用户已绑定角色列表
     *
     * @param request
     * @param userId
     * @return
     */
    @ApiOperation(value = "用户已绑定角色列表")
    @RequestMapping(value = "/get_user_roles", method = RequestMethod.POST)
    public Result getUserRoles(HttpServletRequest request,
                               @RequestParam(value = "userId", required = true) Long userId) {
        List<Long> list = sysRoleService.getUserRoles(userId);
        return ResultUtil.ok(list);
    }
}

